/*
    This file is part of libnspire.

    libnspire is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    libnspire is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with libnspire.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef NSP_KEY_H
#define NSP_KEY_H

#include <inttypes.h>
#include "handle.h"

enum nspire_keys {
    KEYNSP_ESC = 0x1B9600,
    KEYNSP_CTRL_ESC = 0x00EF00,
    KEYNSP_TAB = 0x099500,
    KEYNSP_SHIFT_TAB = 0x7C9500,
    KEYNSP_CTRL_TAB = 0x00ED00,
    KEYNSP_HOME = 0x00FD00,
    KEYNSP_CTRL_HOME = 0x00FE00,
    KEYNSP_MENU = 0x003600,
    KEYNSP_CTRL_MENU = 0x00CE00,
    KEYNSP_MOUSE_CONTEXT_MENU = 0x00A000,
    KEYNSP_CLICK = 0x00AD00,
    KEYNSP_CTRL_CLICK = 0x00AC00,
    KEYNSP_SHIFT_GRAB = 0x00F900,
    KEYNSP_LEFT = 0x000700,
    KEYNSP_SHIFT_LEFT = 0x00F100,
    KEYNSP_SHIFT_HOLD_LEFT = 0x000703,
    KEYNSP_CTRL_LEFT = 0x00DA00,
    KEYNSP_RIGHT = 0x002700,
    KEYNSP_SHIFT_RIGHT = 0x00F200,
    KEYNSP_SHIFT_HOLD_RIGHT = 0x002703,
    KEYNSP_CTRL_RIGHT = 0x00DB00,
    KEYNSP_UP = 0x001700,
    KEYNSP_SHIFT_UP = 0x00F300,
    KEYNSP_SHIFT_HOLD_UP = 0x001703,
    KEYNSP_CTRL_UP = 0x00DC00,
    KEYNSP_DOWN = 0x003700,
    KEYNSP_SHIFT_DOWN = 0x00F400,
    KEYNSP_SHIFT_HOLD_DOWN = 0x003703,
    KEYNSP_CTRL_DOWN = 0x00DD00,
    KEYNSP_EQUAL = 0x3D7500,
    KEYNSP_CTRL_EQUAL = 0x00E900,
    KEYNSP_SUCH_THAT = 0x7CED00,
    KEYNSP_A = 0x616600,
    KEYNSP_SHIFT_A = 0x416600,
    KEYNSP_CTRL_A = 0x616600,
    KEYNSP_B = 0x624600,
    KEYNSP_SHIFT_B = 0x424600,
    KEYNSP_CTRL_B = 0x02B100,
    KEYNSP_C = 0x632600,
    KEYNSP_SHIFT_C = 0x432600,
    KEYNSP_CTRL_C = 0x03B200,
    KEYNSP_FLAG = 0x00A700,
    KEYNSP_CTRL_FLAG = 0x00F800,
    KEYNSP_LESS_THAN = 0x3CA600,
    KEYNSP_CTRL_LESS_THAN = 0x00CF00,
    KEYNSP_D = 0x648500,
    KEYNSP_SHIFT_D = 0x448500,
    KEYNSP_CTRL_D = 0x04B300,
    KEYNSP_E = 0x656500,
    KEYNSP_SHIFT_E = 0x456500,
    KEYNSP_CTRL_E = 0x05B400,
    KEYNSP_F = 0x664500,
    KEYNSP_SHIFT_F = 0x464500,
    KEYNSP_CTRL_F = 0x06B500,
    KEYNSP_G = 0x672500,
    KEYNSP_SHIFT_G = 0x472500,
    KEYNSP_CTRL_G = 0x07B600,
    KEYNSP_TILDE = 0x27F500,
    KEYNSP_CTRL_TILDE = 0x240500,
    KEYNSP_GREATER_THAN = 0x3E8600,
    KEYNSP_CTRL_GREATER_THAN = 0x00DE00,
    KEYNSP_H = 0x688400,
    KEYNSP_SHIFT_H = 0x488400,
    KEYNSP_CTRL_H = 0x08B700,
    KEYNSP_I = 0x696400,
    KEYNSP_SHIFT_I = 0x496400,
    KEYNSP_CTRL_I = 0x09B800,
    KEYNSP_J = 0x6A4400,
    KEYNSP_SHIFT_J = 0x4A4400,
    KEYNSP_CTRL_J = 0x0AB900,
    KEYNSP_K = 0x6B2400,
    KEYNSP_SHIFT_K = 0x4B2400,
    KEYNSP_CTRL_K = 0x0BBA00,
    KEYNSP_QUOTE = 0x22A100,
    KEYNSP_IMAGINARY = 0x00A200,
    KEYNSP_CTRL_IMAGINARY = 0x00A100,
    KEYNSP_L = 0x6C8300,
    KEYNSP_SHIFT_L = 0x4C8300,
    KEYNSP_CTRL_L = 0x0CBB00,
    KEYNSP_M = 0x6D6300,
    KEYNSP_SHIFT_M = 0x4D6300,
    KEYNSP_CTRL_M = 0x0DBC00,
    KEYNSP_N = 0x6E4300,
    KEYNSP_SHIFT_N = 0x4E4300,
    KEYNSP_CTRL_N = 0x0EBD00,
    KEYNSP_O = 0x6F2300,
    KEYNSP_SHIFT_O = 0x4F2300,
    KEYNSP_CTRL_O = 0x0FBE00,
    KEYNSP_COLON = 0x3A0100,
    KEYNSP_CTRL_COLON = 0x3B0200,
    KEYNSP_EXP = 0x00A400,
    KEYNSP_P = 0x708200,
    KEYNSP_SHIFT_P = 0x508200,
    KEYNSP_CTRL_P = 0x10BF00,
    KEYNSP_Q = 0x716200,
    KEYNSP_SHIFT_Q = 0x516200,
    KEYNSP_CTRL_Q = 0x11C000,
    KEYNSP_R = 0x724200,
    KEYNSP_SHIFT_R = 0x524200,
    KEYNSP_CTRL_R = 0x12C100,
    KEYNSP_S = 0x732200,
    KEYNSP_SHIFT_S = 0x532200,
    KEYNSP_CTRL_S = 0x13C200,
    KEYNSP_QUESTION_MARK = 0x3F0300,
    KEYNSP_CTRL_QUESTION_MARK = 0x00CB00,
    KEYNSP_PI = 0x00A300,
    KEYNSP_CTRL_PI = 0x00F400,
    KEYNSP_T = 0x748100,
    KEYNSP_SHIFT_T = 0x548100,
    KEYNSP_CTRL_T = 0x14C300,
    KEYNSP_U = 0x756100,
    KEYNSP_SHIFT_U = 0x556100,
    KEYNSP_CTRL_U = 0x15C400,
    KEYNSP_V = 0x764100,
    KEYNSP_SHIFT_V = 0x564100,
    KEYNSP_CTRL_V = 0x16C500,
    KEYNSP_W = 0x772100,
    KEYNSP_SHIFT_W = 0x572100,
    KEYNSP_CTRL_W = 0x17C600,
    KEYNSP_COMMA = 0x2CA000,
    KEYNSP_THETA = 0x880600,
    KEYNSP_X = 0x788000,
    KEYNSP_SHIFT_X = 0x588000,
    KEYNSP_CTRL_X = 0x18C700,
    KEYNSP_Y = 0x796000,
    KEYNSP_SHIFT_Y = 0x596000,
    KEYNSP_CTRL_Y = 0x19C800,
    KEYNSP_Z = 0x7A4000,
    KEYNSP_SHIFT_Z = 0x5A4000,
    KEYNSP_CTRL_Z = 0x1AC900,
    KEYNSP_SPACE = 0x202000,
    KEYNSP_CTRL_SPACE = 0x00CD00,
    KEYNSP_NEW_LINE = 0x000A00,
    KEYNSP_CTRL = 0x00AA04,
    KEYNSP_SHIFT = 0x00AB03,
    KEYNSP_CTRL_SHIFT = 0x00AB07,
    KEYNSP_BACK_SPACE = 0x081500,
    KEYNSP_SHIFT_BACK_SPACE = 0x081500,
    KEYNSP_CTRL_BACK_SPACE = 0x00E304,
    KEYNSP_VAR = 0x00AF00,
    KEYNSP_CTRL_VAR = 0x00A800,
    KEYNSP_LEFT_PARENTHESES = 0x285500,
    KEYNSP_CTRL_LEFT_PARENTHESES = 0x00E700,
    KEYNSP_RIGHT_PARENTHESES = 0x293500,
    KEYNSP_CTRL_RIGHT_PARENTHESES = 0x00E500,
    KEYNSP_CATALOG = 0x009100,
    KEYNSP_CTRL_CATALOG = 0x00EE00,
    KEYNSP_POWER = 0x5E9300,
    KEYNSP_CTRL_POWER = 0x00EB00,
    KEYNSP_SIN = 0x007400,
    KEYNSP_CTRL_SIN = 0x00E800,
    KEYNSP_COS = 0x005400,
    KEYNSP_CTRL_COS = 0x00E600,
    KEYNSP_TAN = 0x003400,
    KEYNSP_CTRL_TAN = 0x00E400,
    KEYNSP_SLASH = 0x2F1400,
    KEYNSP_CTRL_SLASH = 0x00E200,
    KEYNSP_SQUARE = 0x009300,
    KEYNSP_CTRL_SQUARE = 0x009200,
    KEYNSP_SEVEN = 0x377100,
    KEYNSP_CTRL_SEVEN = 0x00D700,
    KEYNSP_EIGHT = 0x385100,
    KEYNSP_CTRL_EIGHT = 0x00D800,
    KEYNSP_NINE = 0x393100,
    KEYNSP_CTRL_NINE = 0x00D900,
    KEYNSP_TIMES = 0x2A1300,
    KEYNSP_CTRL_TIMES = 0x00E100,
    KEYNSP_TEN_POWER = 0x00EC00,
    KEYNSP_CTRL_TEN_POWER = 0x009400,
    KEYNSP_FOUR = 0x347200,
    KEYNSP_CTRL_FOUR = 0x00D400,
    KEYNSP_FIVE = 0x355200,
    KEYNSP_CTRL_FIVE = 0x00D500,
    KEYNSP_SIX = 0x363200,
    KEYNSP_CTRL_SIX = 0x00D600,
    KEYNSP_MINUS = 0x2D1200,
    KEYNSP_CTRL_MINUS = 0x00E000,
    KEYNSP_E_POWER = 0x00CA00,
    KEYNSP_CTRL_E_POWER = 0x00A900,
    KEYNSP_ONE = 0x317300,
    KEYNSP_CTRL_ONE = 0x00D100,
    KEYNSP_TWO = 0x325300,
    KEYNSP_CTRL_TWO = 0x00D200,
    KEYNSP_THREE = 0x333300,
    KEYNSP_CTRL_THREE = 0x00D300,
    KEYNSP_PLUS = 0x2B1100,
    KEYNSP_CTRL_PLUS = 0x00DF00,
    KEYNSP_ON = 0x000B00,
    KEYNSP_CTRL_ON = 0x00F000,
    KEYNSP_ZERO = 0x305000,
    KEYNSP_CTRL_ZERO = 0x00D000,
    KEYNSP_POINT = 0x2E7000,
    KEYNSP_NEG = 0xB13000,
    KEYNSP_CTRL_NEG = 0x00AE00,
    KEYNSP_ENTER = 0x0D1000,
    KEYNSP_CTRL_ENTER = 0x00A600
};

int nspire_send_key(nspire_handle_t *handle, uint32_t code);
#endif
